// tracer.c - example implementation for the tracer particle example from lecture 3
#include <math.h>
#include "tracer.h"

void tracer_set_parameters(struct Tracer* tracer, float m, float x1, float x2, float qE, float alpha0, float dt)
{
    tracer->m = m;
    tracer->x1 = x1;
    tracer->x2 = x2;
    tracer->qE = qE;
    tracer->alpha0 = alpha0;
    tracer->dt = dt;
}

void tracer_init(struct Tracer* tracer, float* r0,float* v0)
{
    tracer->t = 0.0;   
    for (int c=0; c<DIM; ++c) {
        tracer->r[c] = r0[c];
        tracer->rprev[c] = r0[c] - v0[c]*tracer->dt;
        tracer->f[c] = 0.0;
        tracer->v[c] = v0[c];
    }
}

void tracer_compute_force_and_velocity(struct Tracer* tracer) 
{
    for (int c=0; c<DIM; ++c) {
        tracer->v[c] = (tracer->r[c] - tracer->rprev[c])/tracer->dt;
        tracer->f[c] = -tracer->alpha0*tracer->v[c];
    }
    tracer->f[0] += tracer->qE;
}

void tracer_time_step(struct Tracer *tracer) 
{
    for (int c=0; c<DIM; ++c) {
        float rprev_to_be = tracer->r[c];
        tracer->r[c] = 2*tracer->r[c] - tracer->rprev[c] + tracer->f[c]*tracer->dt*tracer->dt/tracer->m;
        tracer->rprev[c] = rprev_to_be;
    }
    tracer->t += tracer->dt;
    // enforce periodic boundary conditions:
    float L = tracer->x2 - tracer->x1;
    for (int c=0; c<DIM; ++c) {
        while (tracer->r[c] < tracer->x1) {
            tracer->r[c] += L;
            tracer->rprev[c] += L;
        }
        while (tracer->r[c] >= tracer->x2) {
            tracer->r[c] -= L;
            tracer->rprev[c] -= L;
        }
    }
}

void tracer_compute_theory(float* rth, struct Tracer* tracer, float* rinit, float* vinit)
{
    float m_over_alpha0 = tracer->m / tracer->alpha0;
    float qE_over_alpha0 = tracer->qE / tracer->alpha0;
    float expon = exp(-tracer->t/m_over_alpha0);

    for (int c=0; c<DIM; ++c) 
        rth[c] = rinit[c] + vinit[c]*m_over_alpha0*(1-expon);
    rth[0] += qE_over_alpha0 *(tracer->t - m_over_alpha0*(1-expon));

    float L = tracer->x2 - tracer->x1;

    for (int c=0; c<DIM; ++c) {
        while (rth[c] < tracer->x1) 
            rth[c] += L;
        while (rth[c] >= tracer->x2) 
            rth[c] -= L;
    }
}
